@@ -20,25 +20,30 @@
 # THE SOFTWARE.
 """Command line implementation."""
 
+from __future__ import annotations
+
 import errno
 import logging
 import os
 import pathlib
 import shutil
+import site
 import subprocess
 import sys
 from contextlib import contextmanager
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterator, List, Optional
+from typing import TYPE_CHECKING, Any, Callable, Iterator
 
 from ansible_compat.config import ansible_version
 from ansible_compat.prerun import get_cache_dir
 from enrich.console import should_do_markup
+from filelock import FileLock, Timeout
 
 from ansiblelint import cli
+from ansiblelint._mockings import _perform_mockings_cleanup
 from ansiblelint.app import get_app
 from ansiblelint.color import console, console_options, reconfigure, render_yaml
 from ansiblelint.config import options
-from ansiblelint.constants import EXIT_CONTROL_C_RC
+from ansiblelint.constants import EXIT_CONTROL_C_RC, LOCK_TIMEOUT_RC
 from ansiblelint.file_utils import abspath, cwd, normpath
 from ansiblelint.skip_utils import normalize_tag
 from ansiblelint.version import __version__
@@ -79,7 +84,7 @@ def initialize_logger(level: int = 0) -> None:
     _logger.debug("Logging initialized to level %s", logging_level)
 
 
-def initialize_options(arguments: Optional[List[str]] = None) -> None:
+def initialize_options(arguments: list[str] | None = None) -> None:
     """Load config options and store them inside options module."""
     new_options = cli.get_config(arguments or [])
     new_options.cwd = pathlib.Path.cwd()
@@ -103,18 +108,38 @@ def initialize_options(arguments: Optional[List[str]] = None) -> None:
     options.configured = True
     options.cache_dir = get_cache_dir(options.project_dir)
 
+    # add a lock file so we do not have two instances running inside at the same time
+    os.makedirs(options.cache_dir, exist_ok=True)
 
-def _do_list(rules: "RulesCollection") -> int:
+    options.cache_dir_lock = None
+    if not options.offline:
+        options.cache_dir_lock = FileLock(f"{options.cache_dir}/.lock")
+        try:
+            options.cache_dir_lock.acquire(timeout=120)
+        except Timeout:
+            _logger.error(
+                "Timeout waiting for another instance of ansible-lint to release the lock."
+            )
+            sys.exit(LOCK_TIMEOUT_RC)
+
+
+def _do_list(rules: RulesCollection) -> int:
     # On purpose lazy-imports to avoid pre-loading Ansible
     # pylint: disable=import-outside-toplevel
-    from ansiblelint.generate_docs import rules_as_rich, rules_as_rst, rules_as_str
+    from ansiblelint.generate_docs import (
+        rules_as_docs,
+        rules_as_md,
+        rules_as_rich,
+        rules_as_str,
+    )
 
     if options.listrules:
 
-        _rule_format_map: Dict[str, Callable[..., Any]] = {
+        _rule_format_map: dict[str, Callable[..., Any]] = {
             "plain": rules_as_str,
             "rich": rules_as_rich,
-            "rst": rules_as_rst,
+            "md": rules_as_md,
+            "docs": rules_as_docs,
         }
 
         console.print(_rule_format_map[options.format](rules), highlight=False)
@@ -129,7 +154,7 @@ def _do_list(rules: "RulesCollection") -> int:
 
 
 # noinspection PyShadowingNames
-def _do_transform(result: "LintResult", opts: "Namespace") -> None:
+def _do_transform(result: LintResult, opts: Namespace) -> None:
     """Create and run Transformer."""
     if "yaml" in opts.skip_list:
         # The transformer rewrites yaml files, but the user requested to skip
@@ -140,14 +165,14 @@ def _do_transform(result: "LintResult", opts: "Namespace") -> None:
     # pylint: disable=import-outside-toplevel
     from ansiblelint.transformer import Transformer
 
-    # future: maybe pass options to Transformer
-    transformer = Transformer(result)
+    transformer = Transformer(result, options)
 
     # this will mark any matches as fixed if the transforms repaired the issue
     transformer.run()
 
 
-def main(argv: Optional[List[str]] = None) -> int:
+# pylint: disable=too-many-branches
+def main(argv: list[str] | None = None) -> int:  # noqa: C901
     """Linter CLI entry point."""
     # alter PATH if needed (venv support)
     path_inject()
@@ -163,22 +188,27 @@ def main(argv: Optional[List[str]] = None) -> int:
     _logger.debug("Options: %s", options)
     _logger.debug(os.getcwd())
 
-    app = get_app()
+    app = get_app(offline=options.offline)
     # pylint: disable=import-outside-toplevel
     from ansiblelint.rules import RulesCollection
     from ansiblelint.runner import _get_matches
 
-    rules = RulesCollection(options.rulesdirs)
+    rules = RulesCollection(options.rulesdirs, profile=options.profile)
+
+    if options.profile == []:
+        from ansiblelint.generate_docs import profiles_as_rich
+
+        console.print(profiles_as_rich())
+        return 0
 
     if options.listrules or options.listtags:
         return _do_list(rules)
 
     if isinstance(options.tags, str):
         options.tags = options.tags.split(",")
-
     result = _get_matches(rules, options)
 
-    if options.write:
+    if options.write_list:
         _do_transform(result, options)
 
     mark_as_success = False
@@ -216,6 +246,16 @@ def main(argv: Optional[List[str]] = None) -> int:
 
     app.render_matches(result.matches)
 
+    _perform_mockings_cleanup()
+    if options.cache_dir_lock:
+        options.cache_dir_lock.release()
+        pathlib.Path(options.cache_dir_lock.lock_file).unlink(missing_ok=True)
+    if options.mock_filters:
+        _logger.warning(
+            "The following filters were mocked during the run: %s",
+            ",".join(options.mock_filters),
+        )
+
     return app.report_outcome(result, mark_as_success=mark_as_success)
 
 
@@ -229,16 +269,30 @@ def _previous_revision() -> Iterator[None]:
     revision = subprocess.run(
         ["git", "rev-parse", "HEAD^1"],
         check=True,
-        universal_newlines=True,
+        text=True,
         stdout=subprocess.PIPE,
         stderr=subprocess.DEVNULL,
-    ).stdout
+    ).stdout.strip()
+    _logger.info("Previous revision SHA: %s", revision)
     path = pathlib.Path(worktree_dir)
+    if path.exists():
+        shutil.rmtree(worktree_dir)
     path.mkdir(parents=True, exist_ok=True)
-    os.system(f"git worktree add -f {worktree_dir} 2>/dev/null")
+    # Run check will fail if worktree_dir already exists
+    # pylint: disable=subprocess-run-check
+    subprocess.run(
+        ["git", "worktree", "add", "-f", worktree_dir],
+        stdout=subprocess.DEVNULL,
+        stderr=subprocess.DEVNULL,
+    )
     try:
         with cwd(worktree_dir):
-            os.system(f"git checkout {revision}")
+            subprocess.run(
+                ["git", "checkout", revision],
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL,
+                check=True,
+            )
             yield
     finally:
         options.exclude_paths = [abspath(p, os.getcwd()) for p in rel_exclude_paths]
@@ -251,14 +305,14 @@ def _run_cli_entrypoint() -> None:
     """
     try:
         sys.exit(main(sys.argv))
-    except IOError as exc:
+    except OSError as exc:
         # NOTE: Only "broken pipe" is acceptable to ignore
         if exc.errno != errno.EPIPE:
             raise
     except KeyboardInterrupt:
         sys.exit(EXIT_CONTROL_C_RC)
     except RuntimeError as exc:
-        raise SystemExit from exc
+        raise SystemExit(exc) from exc
 
 
 def path_inject() -> None:
@@ -270,16 +324,30 @@ def path_inject() -> None:
     # This must be run before we do run any subprocesses, and loading config
     # does this as part of the ansible detection.
     paths = [x for x in os.environ.get("PATH", "").split(os.pathsep) if x]
-    ansible_path = shutil.which("ansible")
-    if ansible_path:
-        ansible_path = os.path.dirname(ansible_path)
+    inject_paths = []
+
+    userbase_bin_path = f"{site.getuserbase()}/bin"
+    if userbase_bin_path not in paths and os.path.exists(
+        f"{userbase_bin_path}/bin/ansible"
+    ):
+        inject_paths.append(userbase_bin_path)
+
     py_path = os.path.dirname(sys.executable)
-    # Determine if we need to manipulate PATH
-    if ansible_path not in paths or py_path != ansible_path:  # pragma: no cover
-        # tested by test_call_from_outside_venv but coverage cannot detect it
-        paths.insert(0, py_path)
-        os.environ["PATH"] = os.pathsep.join(paths)
-        print(f"WARNING: PATH altered to include {py_path}", file=sys.stderr)
+    if py_path not in paths and os.path.exists(f"{py_path}/ansible"):
+        inject_paths.append(py_path)
+
+    if inject_paths:
+        print(
+            f"WARNING: PATH altered to include {', '.join(inject_paths)} :: This is usually a sign of broken local setup, which can cause unexpected behaviors.",
+            file=sys.stderr,
+        )
+        os.environ["PATH"] = os.pathsep.join([*inject_paths, *paths])
+
+    # We do know that finding ansible in PATH does not guarantee that it is
+    # functioning or that is in fact the same version that was installed as
+    # our dependency, but addressing this would be done by ansible-compat.
+    if not shutil.which("ansible"):
+        raise RuntimeError("Failed to find ansible executable in PATH")
 
 
 if __name__ == "__main__":
