@@ -1,9 +1,11 @@
 """Tests for file utility functions."""
+from __future__ import annotations
+
 import os
 import time
 from argparse import Namespace
 from pathlib import Path
-from typing import Any, Dict, Union
+from typing import Any
 
 import pytest
 from _pytest.capture import CaptureFixture
@@ -19,6 +21,7 @@ from ansiblelint.file_utils import (
     expand_paths_vars,
     guess_project_dir,
     normpath,
+    normpath_path,
 )
 
 from .conftest import cwd
@@ -56,7 +59,7 @@ def test_expand_path_vars(monkeypatch: MonkeyPatch) -> None:
     ),
 )
 def test_expand_paths_vars(
-    test_path: Union[str, Path], expected: str, monkeypatch: MonkeyPatch
+    test_path: str | Path, expected: str, monkeypatch: MonkeyPatch
 ) -> None:
     """Ensure that tilde and env vars are expanded in paths lists."""
     monkeypatch.setenv("TEST_PATH", "/test/path")
@@ -139,6 +142,7 @@ def test_discover_lintables_umlaut(monkeypatch: MonkeyPatch) -> None:
 @pytest.mark.parametrize(
     ("path", "kind"),
     (
+        ("tasks/run_test_playbook.yml", "tasks"),
         ("foo/playbook.yml", "playbook"),
         ("playbooks/foo.yml", "playbook"),
         ("playbooks/roles/foo.yml", "yaml"),
@@ -196,8 +200,8 @@ def test_default_kinds(monkeypatch: MonkeyPatch, path: str, kind: FileType) -> N
     options = cli.get_config([])
 
     # pylint: disable=unused-argument
-    def mockreturn(options: Namespace) -> Dict[str, Any]:
-        return {path: kind}
+    def mockreturn(options: Namespace) -> dict[str, Any]:
+        return {normpath(path): kind}
 
     # assert Lintable is able to determine file type
     lintable_detected = Lintable(path)
@@ -222,7 +226,7 @@ def test_guess_project_dir(tmp_path: Path) -> None:
 BASIC_PLAYBOOK = """
 - name: "playbook"
   tasks:
-    - name: hello
+    - name: Hello
       debug:
         msg: 'world'
 """
@@ -408,3 +412,19 @@ def test_lintable_content_deleter(
     assert tmp_updated_lintable.content == updated_content
     del tmp_updated_lintable.content
     assert tmp_updated_lintable.content == content
+
+
+@pytest.mark.parametrize(
+    ("path", "result"),
+    (
+        pytest.param("foo", "foo", id="rel"),
+        pytest.param(os.path.expanduser("~/xxx"), "~/xxx", id="rel-to-home"),
+        pytest.param("/a/b/c", "/a/b/c", id="absolute"),
+        pytest.param(
+            "examples/playbooks/roles", "examples/roles", id="resolve-symlink"
+        ),
+    ),
+)
+def test_normpath_path(path: str, result: str) -> None:
+    """Tests behavior of normpath."""
+    assert normpath_path(path) == Path(result)
